@@ -9,7 +9,7 @@ module DRb
 	end
         block_value = @block.call(*x)
       end
-      
+
       def perform_with_block
         @obj.__send__(@msg_id, *@argv) do |*x|
           jump_error = nil
@@ -20,8 +20,6 @@ module DRb
           end
           if jump_error
             case jump_error.reason
-            when :retry
-              retry
             when :break
               break(jump_error.exit_value)
             else
