@@ -1,8 +1,7 @@
 #
 #   irb.rb - irb main module
-#   	$Release Version: 0.9.5 $
-#   	$Revision: 24483 $
-#   	$Date: 2009-08-09 17:44:15 +0900 (Sun, 09 Aug 2009) $
+#   	$Release Version: 0.9.6 $
+#   	$Revision: 27117 $
 #   	by Keiju ISHITSUKA(keiju@ruby-lang.org)
 #
 # --
@@ -23,7 +22,7 @@ require "irb/locale"
 STDOUT.sync = true
 
 module IRB
-  @RCS_ID='-$Id: irb.rb 24483 2009-08-09 08:44:15Z shyouhei $-'
+  @RCS_ID='-$Id: irb.rb 27117 2010-03-30 13:03:23Z keiju $-'
 
   class Abort < Exception;end
 
@@ -93,7 +92,7 @@ module IRB
   end
 
   #
-  # irb interpriter main routine 
+  # irb interpreter main routine
   #
   class Irb
     def initialize(workspace = nil, input_method = nil, output_method = nil)
@@ -116,7 +115,7 @@ module IRB
 	  f = @context.prompt_c
 	elsif indent > 0
 	  f = @context.prompt_n
-	else @context.prompt_i
+	else
 	  f = @context.prompt_i
 	end
 	f = "" unless f
@@ -134,7 +133,7 @@ module IRB
 	  end
 	end
       end
-       
+
       @scanner.set_input(@context.io) do
 	signal_status(:IN_INPUT) do
 	  if l = @context.io.gets
@@ -145,6 +144,8 @@ module IRB
 	      if @context.verbose?
 		printf "Use \"exit\" to leave %s\n", @context.ap_name
 	      end
+	    else
+	      print "\n"
 	    end
 	  end
 	  l
@@ -165,12 +166,13 @@ module IRB
 	  end
 	  if exc
 	    print exc.class, ": ", exc, "\n"
-	    if exc.backtrace[0] =~ /irb(2)?(\/.*|-.*|\.rb)?:/ && exc.class.to_s !~ /^IRB/
-	      irb_bug = true 
+	    if exc.backtrace[0] =~ /irb(2)?(\/.*|-.*|\.rb)?:/ && exc.class.to_s !~ /^IRB/ &&
+                !(SyntaxError === exc)
+	      irb_bug = true
 	    else
 	      irb_bug = false
 	    end
-	    
+
 	    messages = []
 	    lasts = []
 	    levels = 0
@@ -182,7 +184,7 @@ module IRB
 		else
 		  lasts.push "\tfrom "+m
 		  if lasts.size > @context.back_trace_limit
-		    lasts.shift 
+		    lasts.shift
 		    levels += 1
 		  end
 		end
@@ -287,13 +289,13 @@ module IRB
 	when "l"
 	  ltype
 	when "i"
-	  if $1 
+	  if $1
 	    format("%" + $1 + "d", indent)
 	  else
 	    indent.to_s
 	  end
 	when "n"
-	  if $1 
+	  if $1
 	    format("%" + $1 + "d", line_no)
 	  else
 	    line_no.to_s
@@ -306,17 +308,13 @@ module IRB
     end
 
     def output_value
-      if @context.inspect?
-        printf @context.return_format, @context.last_value.inspect
-      else
-        printf @context.return_format, @context.last_value
-      end
+      printf @context.return_format, @context.inspect_last_value
     end
 
     def inspect
       ary = []
       for iv in instance_variables
-	case iv
+	case (iv = iv.to_s)
 	when "@signal_status"
 	  ary.push format("%s=:%s", iv, @signal_status.id2name)
 	when "@context"
